# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/take_it_easy',
                'doc_host': 'https://majoma7.github.io',
                'git_url': 'https://github.com/majoma7/take_it_easy',
                'lib_path': 'take_it_easy'},
  'syms': { 'take_it_easy.board': { 'take_it_easy.board.Board': ('board.html#board', 'take_it_easy/board.py'),
                                    'take_it_easy.board.Board.__init__': ('board.html#board.__init__', 'take_it_easy/board.py'),
                                    'take_it_easy.board.Board.__str__': ('board.html#board.__str__', 'take_it_easy/board.py'),
                                    'take_it_easy.board.Board.get_column_score': ( 'board.html#board.get_column_score',
                                                                                   'take_it_easy/board.py'),
                                    'take_it_easy.board.Board.get_index': ('board.html#board.get_index', 'take_it_easy/board.py'),
                                    'take_it_easy.board.Board.get_potential_moves': ( 'board.html#board.get_potential_moves',
                                                                                      'take_it_easy/board.py'),
                                    'take_it_easy.board.Board.get_score': ('board.html#board.get_score', 'take_it_easy/board.py'),
                                    'take_it_easy.board.Board.get_score_potential': ( 'board.html#board.get_score_potential',
                                                                                      'take_it_easy/board.py'),
                                    'take_it_easy.board.Board.is_full': ('board.html#board.is_full', 'take_it_easy/board.py'),
                                    'take_it_easy.board.Board.put_tile': ('board.html#board.put_tile', 'take_it_easy/board.py'),
                                    'take_it_easy.board.Board.reset': ('board.html#board.reset', 'take_it_easy/board.py')},
            'take_it_easy.play': { 'take_it_easy.play.Game': ('play.html#game', 'take_it_easy/play.py'),
                                   'take_it_easy.play.Game.__init__': ('play.html#game.__init__', 'take_it_easy/play.py'),
                                   'take_it_easy.play.Game.reset': ('play.html#game.reset', 'take_it_easy/play.py'),
                                   'take_it_easy.play.Game.step': ('play.html#game.step', 'take_it_easy/play.py'),
                                   'take_it_easy.play.play_random_game': ('play.html#play_random_game', 'take_it_easy/play.py')},
            'take_it_easy.tiles': { 'take_it_easy.tiles.Tiles': ('tiles.html#tiles', 'take_it_easy/tiles.py'),
                                    'take_it_easy.tiles.Tiles.__init__': ('tiles.html#tiles.__init__', 'take_it_easy/tiles.py'),
                                    'take_it_easy.tiles.Tiles.__str__': ('tiles.html#tiles.__str__', 'take_it_easy/tiles.py'),
                                    'take_it_easy.tiles.Tiles.draw_tile': ('tiles.html#tiles.draw_tile', 'take_it_easy/tiles.py'),
                                    'take_it_easy.tiles.Tiles.input_tile': ('tiles.html#tiles.input_tile', 'take_it_easy/tiles.py'),
                                    'take_it_easy.tiles.Tiles.reset': ('tiles.html#tiles.reset', 'take_it_easy/tiles.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/02_board.ipynb.

# %% auto 0
__all__ = ['Board']

# %% ../nbs/02_board.ipynb 2
import numpy as np

# %% ../nbs/02_board.ipynb 4
class Board():

    '''A class to represent the board. The board is a hexacon with 19 tiles that are represented with cubic coordinates'''

    def __init__(self,
                 ):
        '''Initialize the board.'''
        # Create the board
        self.board = np.zeros((19, 6), dtype=int)
        # Create the board coordinates using the first three columns
        single_index = 0
        for q in range (-2, 3):
            for s in reversed(range(-2, 3)):
                r = -q - s
                # pass if maximum absolute of any variable value is greater than 2
                if max(abs(q), abs(s), abs(r)) > 2:
                    continue
                self.board[single_index, 0:3] = [q, r, s]
                single_index += 1
        self.potential_moves=np.ones(19, dtype=bool)


    def __str__(self):
        return "Tiles object"
    
    __repr__ = __str__
    
    def put_tile(self, tile, coordinates):
        '''Put a tile on the board.'''
        # Get the index of the coordinates
        index = self.get_index(coordinates)
        if self.board[index, 3] != 0:
            return("Slot already occupied")
        else:
            # Put the tile on the board
            self.board[index, 3:] = tile
            self.potential_moves[index] = False
            return("Tile placed")

    
    def get_index(self, coordinates):
        '''Get the index of the coordinates.'''
        # Get the index of the coordinates
        index = np.where((self.board[:, 0] == coordinates[0]) & (self.board[:, 1] == coordinates[1]) & (self.board[:, 2] == coordinates[2]))
        return index
    
    def get_column_score(self, column_index, potential=False):
        value_columns=0
        for i in range(-2, 3):
            column = self.board[self.board[:, column_index]==i, column_index+3]
            if potential:
                non_zero_column = column[column != 0]
                if all (x == non_zero_column[0] for x in non_zero_column):
                    value_column = np.sum(column)
                else:
                    value_column = 0
            else:
                if all (x == column[0] for x in column):
                    value_column = np.sum(column)
                else:
                    value_column = 0
            value_columns += value_column
        return value_columns
    
    def get_score(self):
        '''Get the score of the board.'''
        score = 0
        for i in range(3):
            score += self.get_column_score(i, potential=False)
        return score
    
    def get_score_potential(self):
        '''Get the score of the board.'''
        score = 0
        for i in range(3):
            score += self.get_column_score(i, potential=True)
        return score
    
    def get_potential_moves(self):
        '''Get the potential moves.'''
        indices = np.where(self.potential_moves)[0]
        potential_moves_coordinates = self.board[indices, :3].copy()
        return self.potential_moves.copy(), potential_moves_coordinates

    
    def is_full(self):
        '''Check if the board is full.'''
        if sum(self.potential_moves) == 0:
            return True
        else:
            return False

    def reset(self):
        '''Reset the chosen array to all False.'''
        self.board[:, 3:]=0



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/03_play.ipynb.

# %% auto 0
__all__ = ['Game', 'play_random_game']

# %% ../nbs/03_play.ipynb 2
import numpy as np

# %% ../nbs/03_play.ipynb 4
class Game():
    """
    Game class for playing the game.
    """
    def __init__(self, board, tiles):
        self.board = board
        self.tiles = tiles
        self.finished = False

    def step(self, tile, action):
        
        """
        Do one step.
        """
        score_beginning = self.board.get_score()
        score_potential_beginning = self.board.get_score_potential()

        self.board.put_tile(tile, action)

        next_tile = self.tiles.draw_tile()

        indices, coordinates = self.board.get_potential_moves()

        score_change = self.board.get_score() - score_beginning
        score_potential_change = self.board.get_score_potential() - score_potential_beginning

        if self.board.is_full():
            self.finished = True

        return indices, coordinates, score_change, score_potential_change, next_tile

    def reset(self):
        self.board.reset()
        self.tiles.reset()
        self.finished = False

def play_random_game(board, tiles):
    """
    Play the game board.
    """
    game = Game(board, tiles)
    indices, coordinates = game.board.get_potential_moves()
    tile = game.tiles.draw_tile()

    while not game.finished:
        action = np.random.choice(np.array(len(coordinates[:, 0])))
        action = coordinates[action, :]
        indices, coordinates, score_change, score_potential_change, tile = game.step(tile, action)
        print(f"Score impact: {score_change}, potential score impact: {score_potential_change}")
    print("finished game with score: ", game.board.get_score())



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_tiles.ipynb.

# %% auto 0
__all__ = ['Tiles']

# %% ../nbs/01_tiles.ipynb 3
import numpy as np

# %% ../nbs/01_tiles.ipynb 4
class Tiles():

    '''Class to store all tiles and whether they have been drawn or not.
    Also manages the drawing mechanism.'''

    def __init__(self,
                 random_seed = 0, # random seed for the drawing mechanism
                 mode = "play" # "play" or "input": "play" mode draws a random tile, "input" mode if the user inputs the tile that has been drawn
                 ):

        digits = np.array([[1, 5, 9], [3, 4, 8], [2, 6, 7]])
        self.chips =np.zeros((27, 3), dtype=int)

        for i in range(27):
            self.chips[i] = [digits[0][i//9], digits[1][(i//3)%3], digits[2][i%3]]

        # Initialize boolean array to all False
        self.chosen = np.zeros(27, dtype=bool)
        self.mode = mode

    def __str__(self):
        return "Tiles object"
    
    __repr__ = __str__

    def draw_tile(self):
        assert self.mode == "play", "You are in input mode, you need to input the tile that has been drawn."
        '''Draw a random tile from the remaining tiles.'''
        # Get the indices of the remaining tiles
        remaining = np.where(self.chosen == False)[0]
        # Draw a random tile
        draw = np.random.choice(remaining)
        # Mark the tile as drawn
        self.chosen[draw] = True
        return self.chips[draw]

    def input_tile(self, tile):
        assert self.mode == "input", "You are in play mode, you need to draw a tile."
        '''Input the tile that has been drawn.'''
        # Find the index of the drawn tile
        draw = np.where((self.chips == tile).all(axis=1))[0][0]
        # Mark the tile as drawn
        self.chosen[draw] = True
        return self.chips[draw]
    
    def reset(self):
        '''Reset the chosen array to all False.'''
        self.chosen = np.zeros(27, dtype=bool)
